@@ -1498,7 +1498,7 @@ static int WaitRecv(UserData* udata)
 // -----------------------------------------------------------------------------
 
 // Initialize memory allocated within Userdata
-UserData::UserData(sundials::Context& sunctx) : sunctx(sunctx)
+UserData::UserData(sundials::Context& ctx) : sunctx(ctx)
 {
   // Diffusion coefficient
   kx = ONE;
@@ -1932,8 +1932,8 @@ static int OpenOutput(UserData* udata)
 static int WriteOutput(sunrealtype t, N_Vector u, UserData* udata)
 {
   int flag;
-  sunrealtype max;
-  bool outproc = (udata->myid_c == 0);
+  sunrealtype max = ZERO;
+  bool outproc    = (udata->myid_c == 0);
 
   if (udata->output > 0)
   {
