@@ -336,8 +336,8 @@ static int arkRelaxBrentSolve(ARKodeMem ark_mem)
 }
 
 /* Compute and apply relaxation parameter */
-int arkRelaxSolve(ARKodeMem ark_mem, ARKodeRelaxMem relax_mem,
-                  sunrealtype* relax_val_out)
+static int arkRelaxSolve(ARKodeMem ark_mem, ARKodeRelaxMem relax_mem,
+                         sunrealtype* relax_val_out)
 {
   int retval;
 
@@ -357,7 +357,7 @@ int arkRelaxSolve(ARKodeMem ark_mem, ARKodeRelaxMem relax_mem,
 
 #ifdef SUNDIALS_LOGGING_EXTRA_DEBUG
   SUNLogger_QueueMsg(ARK_LOGGER, SUN_LOGLEVEL_DEBUG, "ARKODE::arkRelaxSolve",
-                     "compute delta y", "delta_y =", "");
+                     "compute delta y", "delta_y(:) =", "");
   N_VPrintFile(ark_mem->tempv2, ARK_LOGGER->debug_fp);
 #endif
 
@@ -415,7 +415,31 @@ int arkRelaxSolve(ARKodeMem ark_mem, ARKodeRelaxMem relax_mem,
  * Set functions
  * ---------------------------------------------------------------------------*/
 
-int arkRelaxSetEtaFail(void* arkode_mem, sunrealtype eta_fail)
+int ARKodeSetRelaxFn(void* arkode_mem, ARKRelaxFn rfn, ARKRelaxJacFn rjac)
+{
+  ARKodeMem ark_mem;
+  if (arkode_mem == NULL)
+  {
+    arkProcessError(NULL, ARK_MEM_NULL, __LINE__, __func__, __FILE__,
+                    MSG_ARK_NO_MEM);
+    return (ARK_MEM_NULL);
+  }
+  ark_mem = (ARKodeMem)arkode_mem;
+
+  /* Call stepper-specific routine (if it exists) */
+  if (ark_mem->step_setrelaxfn)
+  {
+    return ark_mem->step_setrelaxfn(arkode_mem, rfn, rjac);
+  }
+  else
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+}
+
+int ARKodeSetRelaxEtaFail(void* arkode_mem, sunrealtype eta_fail)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -424,13 +448,21 @@ int arkRelaxSetEtaFail(void* arkode_mem, sunrealtype eta_fail)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   if (eta_fail > ZERO && eta_fail < ONE) { relax_mem->eta_fail = eta_fail; }
   else { relax_mem->eta_fail = ARK_RELAX_DEFAULT_ETA_FAIL; }
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxSetLowerBound(void* arkode_mem, sunrealtype lower)
+int ARKodeSetRelaxLowerBound(void* arkode_mem, sunrealtype lower)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -439,13 +471,21 @@ int arkRelaxSetLowerBound(void* arkode_mem, sunrealtype lower)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   if (lower > ZERO && lower < ONE) { relax_mem->lower_bound = lower; }
   else { relax_mem->lower_bound = ARK_RELAX_DEFAULT_LOWER_BOUND; }
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxSetMaxFails(void* arkode_mem, int max_fails)
+int ARKodeSetRelaxMaxFails(void* arkode_mem, int max_fails)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -454,13 +494,21 @@ int arkRelaxSetMaxFails(void* arkode_mem, int max_fails)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   if (max_fails > 0) { relax_mem->max_fails = max_fails; }
   else { relax_mem->max_fails = ARK_RELAX_DEFAULT_MAX_FAILS; }
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxSetMaxIters(void* arkode_mem, int max_iters)
+int ARKodeSetRelaxMaxIters(void* arkode_mem, int max_iters)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -469,13 +517,21 @@ int arkRelaxSetMaxIters(void* arkode_mem, int max_iters)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   if (max_iters > 0) { relax_mem->max_iters = max_iters; }
   else { relax_mem->max_iters = ARK_RELAX_DEFAULT_MAX_ITERS; }
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxSetSolver(void* arkode_mem, ARKRelaxSolver solver)
+int ARKodeSetRelaxSolver(void* arkode_mem, ARKRelaxSolver solver)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -484,6 +540,14 @@ int arkRelaxSetSolver(void* arkode_mem, ARKRelaxSolver solver)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   if (solver != ARK_RELAX_BRENT && solver != ARK_RELAX_NEWTON)
   {
     arkProcessError(ark_mem, ARK_ILL_INPUT, __LINE__, __func__, __FILE__,
@@ -496,7 +560,7 @@ int arkRelaxSetSolver(void* arkode_mem, ARKRelaxSolver solver)
   return ARK_SUCCESS;
 }
 
-int arkRelaxSetResTol(void* arkode_mem, sunrealtype res_tol)
+int ARKodeSetRelaxResTol(void* arkode_mem, sunrealtype res_tol)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -505,13 +569,21 @@ int arkRelaxSetResTol(void* arkode_mem, sunrealtype res_tol)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   if (res_tol > ZERO) { relax_mem->res_tol = res_tol; }
   else { relax_mem->res_tol = ARK_RELAX_DEFAULT_RES_TOL; }
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxSetTol(void* arkode_mem, sunrealtype rel_tol, sunrealtype abs_tol)
+int ARKodeSetRelaxTol(void* arkode_mem, sunrealtype rel_tol, sunrealtype abs_tol)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -520,6 +592,14 @@ int arkRelaxSetTol(void* arkode_mem, sunrealtype rel_tol, sunrealtype abs_tol)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   if (rel_tol > ZERO) { relax_mem->rel_tol = rel_tol; }
   else { relax_mem->rel_tol = ARK_RELAX_DEFAULT_REL_TOL; }
 
@@ -529,7 +609,7 @@ int arkRelaxSetTol(void* arkode_mem, sunrealtype rel_tol, sunrealtype abs_tol)
   return ARK_SUCCESS;
 }
 
-int arkRelaxSetUpperBound(void* arkode_mem, sunrealtype upper)
+int ARKodeSetRelaxUpperBound(void* arkode_mem, sunrealtype upper)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -538,6 +618,14 @@ int arkRelaxSetUpperBound(void* arkode_mem, sunrealtype upper)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   if (upper > ONE) { relax_mem->upper_bound = upper; }
   else { relax_mem->upper_bound = ARK_RELAX_DEFAULT_UPPER_BOUND; }
 
@@ -548,7 +636,7 @@ int arkRelaxSetUpperBound(void* arkode_mem, sunrealtype upper)
  * Get functions
  * ---------------------------------------------------------------------------*/
 
-int arkRelaxGetNumRelaxFnEvals(void* arkode_mem, long int* r_evals)
+int ARKodeGetNumRelaxFnEvals(void* arkode_mem, long int* r_evals)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -557,12 +645,20 @@ int arkRelaxGetNumRelaxFnEvals(void* arkode_mem, long int* r_evals)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   *r_evals = relax_mem->num_relax_fn_evals;
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxGetNumRelaxJacEvals(void* arkode_mem, long int* J_evals)
+int ARKodeGetNumRelaxJacEvals(void* arkode_mem, long int* J_evals)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -571,12 +667,20 @@ int arkRelaxGetNumRelaxJacEvals(void* arkode_mem, long int* J_evals)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   *J_evals = relax_mem->num_relax_jac_evals;
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxGetNumRelaxFails(void* arkode_mem, long int* relax_fails)
+int ARKodeGetNumRelaxFails(void* arkode_mem, long int* relax_fails)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -585,12 +689,20 @@ int arkRelaxGetNumRelaxFails(void* arkode_mem, long int* relax_fails)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   *relax_fails = relax_mem->num_fails;
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxGetNumRelaxSolveFails(void* arkode_mem, long int* fails)
+int ARKodeGetNumRelaxSolveFails(void* arkode_mem, long int* fails)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -599,12 +711,20 @@ int arkRelaxGetNumRelaxSolveFails(void* arkode_mem, long int* fails)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   *fails = relax_mem->nls_fails;
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxGetNumRelaxBoundFails(void* arkode_mem, long int* fails)
+int ARKodeGetNumRelaxBoundFails(void* arkode_mem, long int* fails)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -613,12 +733,20 @@ int arkRelaxGetNumRelaxBoundFails(void* arkode_mem, long int* fails)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   *fails = relax_mem->bound_fails;
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxGetNumRelaxSolveIters(void* arkode_mem, long int* iters)
+int ARKodeGetNumRelaxSolveIters(void* arkode_mem, long int* iters)
 {
   int retval;
   ARKodeMem ark_mem;
@@ -627,73 +755,28 @@ int arkRelaxGetNumRelaxSolveIters(void* arkode_mem, long int* iters)
   retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
   if (retval) { return retval; }
 
+  /* Guard against use for time steppers that do not allow relaxation */
+  if (!ark_mem->step_supports_relaxation)
+  {
+    arkProcessError(ark_mem, ARK_STEPPER_UNSUPPORTED, __LINE__, __func__,
+                    __FILE__, "time-stepping module does not support relaxation");
+    return (ARK_STEPPER_UNSUPPORTED);
+  }
+
   *iters = relax_mem->nls_iters;
 
   return ARK_SUCCESS;
 }
 
-int arkRelaxPrintAllStats(void* arkode_mem, FILE* outfile, SUNOutputFormat fmt)
-{
-  int retval;
-  ARKodeMem ark_mem;
-  ARKodeRelaxMem relax_mem;
-
-  retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
-  if (retval) { return retval; }
-
-  switch (fmt)
-  {
-  case SUN_OUTPUTFORMAT_TABLE:
-    fprintf(outfile, "Relax fn evals               = %ld\n",
-            relax_mem->num_relax_fn_evals);
-    fprintf(outfile, "Relax Jac evals              = %ld\n",
-            relax_mem->num_relax_jac_evals);
-    fprintf(outfile, "Relax fails                  = %ld\n",
-            relax_mem->num_fails);
-    fprintf(outfile, "Relax bound fails            = %ld\n",
-            relax_mem->bound_fails);
-    fprintf(outfile, "Relax NLS iters              = %ld\n",
-            relax_mem->nls_iters);
-    fprintf(outfile, "Relax NLS fails              = %ld\n",
-            relax_mem->nls_fails);
-    break;
-  case SUN_OUTPUTFORMAT_CSV:
-    fprintf(outfile, ",Relax fn evals,%ld", relax_mem->num_relax_fn_evals);
-    fprintf(outfile, ",Relax Jac evals,%ld", relax_mem->num_relax_jac_evals);
-    fprintf(outfile, ",Relax fails,%ld", relax_mem->num_fails);
-    fprintf(outfile, ",Relax bound fails,%ld", relax_mem->bound_fails);
-    fprintf(outfile, ",Relax NLS iters,%ld", relax_mem->nls_iters);
-    fprintf(outfile, ",Relax NLS fails,%ld", relax_mem->nls_fails);
-    break;
-  default:
-    arkProcessError(ark_mem, ARK_ILL_INPUT, __LINE__, __func__, __FILE__,
-                    "Invalid formatting option.");
-    return ARK_ILL_INPUT;
-  }
-
-  return ARK_SUCCESS;
-}
-
 /* =============================================================================
  * Driver and Stepper Functions
  * ===========================================================================*/
 
 /* Constructor called by stepper */
-int arkRelaxCreate(void* arkode_mem, ARKRelaxFn relax_fn,
+int arkRelaxCreate(ARKodeMem ark_mem, ARKRelaxFn relax_fn,
                    ARKRelaxJacFn relax_jac_fn, ARKRelaxDeltaEFn delta_e_fn,
                    ARKRelaxGetOrderFn get_order_fn)
 {
-  ARKodeMem ark_mem;
-
-  /* Check inputs */
-  if (!arkode_mem)
-  {
-    arkProcessError(NULL, ARK_MEM_NULL, __LINE__, __func__, __FILE__,
-                    MSG_ARK_NO_MEM);
-    return ARK_MEM_NULL;
-  }
-  ark_mem = (ARKodeMem)arkode_mem;
-
   /* Disable relaxation if both user inputs are NULL */
   if (!relax_fn && !relax_jac_fn)
   {
@@ -774,8 +857,7 @@ int arkRelaxDestroy(ARKodeRelaxMem relax_mem)
 }
 
 /* Compute and apply relaxation, called by driver */
-int arkRelax(ARKodeMem ark_mem, int* relax_fails, sunrealtype* dsm_inout,
-             int* nflag_out)
+int arkRelax(ARKodeMem ark_mem, int* relax_fails, sunrealtype* dsm_inout)
 {
   int retval;
   sunrealtype relax_val;
@@ -841,6 +923,49 @@ int arkRelax(ARKodeMem ark_mem, int* relax_fails, sunrealtype* dsm_inout,
   return ARK_SUCCESS;
 }
 
+/* Print relaxation solver statistics, called by ARKODE */
+int arkRelaxPrintAllStats(void* arkode_mem, FILE* outfile, SUNOutputFormat fmt)
+{
+  int retval;
+  ARKodeMem ark_mem;
+  ARKodeRelaxMem relax_mem;
+
+  retval = arkRelaxAccessMem(arkode_mem, __func__, &ark_mem, &relax_mem);
+  if (retval) { return retval; }
+
+  switch (fmt)
+  {
+  case SUN_OUTPUTFORMAT_TABLE:
+    fprintf(outfile, "Relax fn evals               = %ld\n",
+            relax_mem->num_relax_fn_evals);
+    fprintf(outfile, "Relax Jac evals              = %ld\n",
+            relax_mem->num_relax_jac_evals);
+    fprintf(outfile, "Relax fails                  = %ld\n",
+            relax_mem->num_fails);
+    fprintf(outfile, "Relax bound fails            = %ld\n",
+            relax_mem->bound_fails);
+    fprintf(outfile, "Relax NLS iters              = %ld\n",
+            relax_mem->nls_iters);
+    fprintf(outfile, "Relax NLS fails              = %ld\n",
+            relax_mem->nls_fails);
+    break;
+  case SUN_OUTPUTFORMAT_CSV:
+    fprintf(outfile, ",Relax fn evals,%ld", relax_mem->num_relax_fn_evals);
+    fprintf(outfile, ",Relax Jac evals,%ld", relax_mem->num_relax_jac_evals);
+    fprintf(outfile, ",Relax fails,%ld", relax_mem->num_fails);
+    fprintf(outfile, ",Relax bound fails,%ld", relax_mem->bound_fails);
+    fprintf(outfile, ",Relax NLS iters,%ld", relax_mem->nls_iters);
+    fprintf(outfile, ",Relax NLS fails,%ld", relax_mem->nls_fails);
+    break;
+  default:
+    arkProcessError(ark_mem, ARK_ILL_INPUT, __LINE__, __func__, __FILE__,
+                    "Invalid formatting option.");
+    return ARK_ILL_INPUT;
+  }
+
+  return ARK_SUCCESS;
+}
+
 /* =============================================================================
  * EOF
  * ===========================================================================*/
