@@ -23,7 +23,7 @@
 #include "arkode/arkode_butcher.h"
 #include "arkode_impl.h"
 
-ARKodeSPRKTable ARKodeSymplecticEuler(void)
+static ARKodeSPRKTable arkodeSymplecticEuler(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(1);
   if (!sprk_table) { return NULL; }
@@ -43,7 +43,7 @@ ARKodeSPRKTable ARKodeSymplecticEuler(void)
   https://doi.org/10.1016/0021-9991(91)90299-Z.
  */
 
-ARKodeSPRKTable ARKodeSymplecticLeapfrog2(void)
+static ARKodeSPRKTable arkodeSymplecticLeapfrog2(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(2);
   if (!sprk_table) { return NULL; }
@@ -56,7 +56,7 @@ ARKodeSPRKTable ARKodeSymplecticLeapfrog2(void)
   return sprk_table;
 }
 
-ARKodeSPRKTable ARKodeSymplecticPseudoLeapfrog2(void)
+static ARKodeSPRKTable arkodeSymplecticPseudoLeapfrog2(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(2);
   if (!sprk_table) { return NULL; }
@@ -69,7 +69,7 @@ ARKodeSPRKTable ARKodeSymplecticPseudoLeapfrog2(void)
   return sprk_table;
 }
 
-ARKodeSPRKTable ARKodeSymplecticCandyRozmus4(void)
+static ARKodeSPRKTable arkodeSymplecticCandyRozmus4(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(4);
   if (!sprk_table) { return NULL; }
@@ -108,7 +108,7 @@ ARKodeSPRKTable ARKodeSymplecticCandyRozmus4(void)
   https://accelconf.web.cern.ch/p83/PDF/PAC1983_2669.PDF
  */
 
-ARKodeSPRKTable ARKodeSymplecticRuth3(void)
+static ARKodeSPRKTable arkodeSymplecticRuth3(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(3);
   if (!sprk_table) { return NULL; }
@@ -130,14 +130,14 @@ ARKodeSPRKTable ARKodeSymplecticRuth3(void)
   Nonlinearity. 5, 541–562 (1992). https://doi.org/10.1088/0951-7715/5/2/011
  */
 
-ARKodeSPRKTable ARKodeSymplecticMcLachlan2(void)
+static ARKodeSPRKTable arkodeSymplecticMcLachlan2(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(2);
   if (!sprk_table) { return NULL; }
   sprk_table->q      = 2;
   sprk_table->stages = 2;
-  sprk_table->a[1]   = SUN_RCONST(1.0) -
-                     (SUN_RCONST(1.0) / SUN_RCONST(2.0)) * SUNRsqrt(2.0);
+  sprk_table->a[1]   = SUN_RCONST(1.0) - (SUN_RCONST(1.0) / SUN_RCONST(2.0)) *
+                                         SUNRsqrt(SUN_RCONST(2.0));
   sprk_table->a[0]    = SUN_RCONST(1.0) - sprk_table->a[1];
   sprk_table->ahat[1] = SUN_RCONST(1.0) /
                         (SUN_RCONST(2.0) * (SUN_RCONST(1.0) - sprk_table->a[1]));
@@ -145,11 +145,11 @@ ARKodeSPRKTable ARKodeSymplecticMcLachlan2(void)
   return sprk_table;
 }
 
-ARKodeSPRKTable ARKodeSymplecticMcLachlan3(void)
+static ARKodeSPRKTable arkodeSymplecticMcLachlan3(void)
 {
-  sunrealtype w              = 0.0;
-  sunrealtype y              = 0.0;
-  sunrealtype z              = 0.0;
+  sunrealtype w              = SUN_RCONST(0.0);
+  sunrealtype y              = SUN_RCONST(0.0);
+  sunrealtype z              = SUN_RCONST(0.0);
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(3);
   if (!sprk_table) { return NULL; }
 
@@ -157,7 +157,8 @@ ARKodeSPRKTable ARKodeSymplecticMcLachlan3(void)
   sprk_table->stages = 3;
 
   z = -SUNRpowerR((SUN_RCONST(2.0) / SUN_RCONST(27.0)) -
-                    SUN_RCONST(1.0) / (SUN_RCONST(9.0) * SUNRsqrt(3.0)),
+                    SUN_RCONST(1.0) /
+                      (SUN_RCONST(9.0) * SUNRsqrt(SUN_RCONST(3.0))),
                   SUN_RCONST(1.0) / SUN_RCONST(3.0));
   w = -SUN_RCONST(2.0) / SUN_RCONST(3.0) +
       SUN_RCONST(1.0) / (SUN_RCONST(9.0) * z) + z;
@@ -174,7 +175,7 @@ ARKodeSPRKTable ARKodeSymplecticMcLachlan3(void)
   return sprk_table;
 }
 
-ARKodeSPRKTable ARKodeSymplecticMcLachlan4(void)
+static ARKodeSPRKTable arkodeSymplecticMcLachlan4(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(4);
   if (!sprk_table) { return NULL; }
@@ -191,7 +192,7 @@ ARKodeSPRKTable ARKodeSymplecticMcLachlan4(void)
   return sprk_table;
 }
 
-ARKodeSPRKTable ARKodeSymplecticMcLachlan5(void)
+static ARKodeSPRKTable arkodeSymplecticMcLachlan5(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(6);
   if (!sprk_table) { return NULL; }
@@ -221,7 +222,7 @@ ARKodeSPRKTable ARKodeSymplecticMcLachlan5(void)
 
  */
 
-ARKodeSPRKTable ARKodeSymplecticYoshida6(void)
+static ARKodeSPRKTable arkodeSymplecticYoshida6(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(8);
   if (!sprk_table) { return NULL; }
@@ -260,7 +261,7 @@ ARKodeSPRKTable ARKodeSymplecticYoshida6(void)
 
  */
 
-ARKodeSPRKTable ARKodeSymplecticSuzukiUmeno816(void)
+static ARKodeSPRKTable arkodeSymplecticSuzukiUmeno816(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(16);
   if (!sprk_table) { return NULL; }
@@ -310,7 +311,7 @@ ARKodeSPRKTable ARKodeSymplecticSuzukiUmeno816(void)
 
  */
 
-ARKodeSPRKTable ARKodeSymplecticSofroniou10(void)
+static ARKodeSPRKTable arkodeSymplecticSofroniou10(void)
 {
   ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(36);
   if (!sprk_table) { return NULL; }
@@ -403,16 +404,15 @@ ARKodeSPRKTable ARKodeSymplecticSofroniou10(void)
 ARKodeSPRKTable ARKodeSPRKTable_Create(int s, int q, const sunrealtype* a,
                                        const sunrealtype* ahat)
 {
-  int i                      = 0;
-  ARKodeSPRKTable sprk_table = NULL;
+  if (s < 1 || !a || !ahat) { return NULL; }
 
-  sprk_table = (ARKodeSPRKTable)malloc(sizeof(struct ARKodeSPRKTableMem));
+  ARKodeSPRKTable sprk_table = ARKodeSPRKTable_Alloc(s);
   if (!sprk_table) { return NULL; }
 
   sprk_table->stages = s;
   sprk_table->q      = q;
 
-  for (i = 0; i < s; i++)
+  for (int i = 0; i < s; i++)
   {
     sprk_table->a[i]    = a[i];
     sprk_table->ahat[i] = ahat[i];
@@ -453,19 +453,19 @@ ARKodeSPRKTable ARKodeSPRKTable_Load(ARKODE_SPRKMethodID id)
 {
   switch (id)
   {
-  case ARKODE_SPRK_EULER_1_1: return ARKodeSymplecticEuler();
-  case ARKODE_SPRK_LEAPFROG_2_2: return ARKodeSymplecticLeapfrog2();
+  case ARKODE_SPRK_EULER_1_1: return arkodeSymplecticEuler();
+  case ARKODE_SPRK_LEAPFROG_2_2: return arkodeSymplecticLeapfrog2();
   case ARKODE_SPRK_PSEUDO_LEAPFROG_2_2:
-    return ARKodeSymplecticPseudoLeapfrog2();
-  case ARKODE_SPRK_RUTH_3_3: return ARKodeSymplecticRuth3();
-  case ARKODE_SPRK_MCLACHLAN_2_2: return ARKodeSymplecticMcLachlan2();
-  case ARKODE_SPRK_MCLACHLAN_3_3: return ARKodeSymplecticMcLachlan3();
-  case ARKODE_SPRK_MCLACHLAN_4_4: return ARKodeSymplecticMcLachlan4();
-  case ARKODE_SPRK_CANDY_ROZMUS_4_4: return ARKodeSymplecticCandyRozmus4();
-  case ARKODE_SPRK_MCLACHLAN_5_6: return ARKodeSymplecticMcLachlan5();
-  case ARKODE_SPRK_YOSHIDA_6_8: return ARKodeSymplecticYoshida6();
-  case ARKODE_SPRK_SUZUKI_UMENO_8_16: return ARKodeSymplecticSuzukiUmeno816();
-  case ARKODE_SPRK_SOFRONIOU_10_36: return ARKodeSymplecticSofroniou10();
+    return arkodeSymplecticPseudoLeapfrog2();
+  case ARKODE_SPRK_RUTH_3_3: return arkodeSymplecticRuth3();
+  case ARKODE_SPRK_MCLACHLAN_2_2: return arkodeSymplecticMcLachlan2();
+  case ARKODE_SPRK_MCLACHLAN_3_3: return arkodeSymplecticMcLachlan3();
+  case ARKODE_SPRK_MCLACHLAN_4_4: return arkodeSymplecticMcLachlan4();
+  case ARKODE_SPRK_CANDY_ROZMUS_4_4: return arkodeSymplecticCandyRozmus4();
+  case ARKODE_SPRK_MCLACHLAN_5_6: return arkodeSymplecticMcLachlan5();
+  case ARKODE_SPRK_YOSHIDA_6_8: return arkodeSymplecticYoshida6();
+  case ARKODE_SPRK_SUZUKI_UMENO_8_16: return arkodeSymplecticSuzukiUmeno816();
+  case ARKODE_SPRK_SOFRONIOU_10_36: return arkodeSymplecticSofroniou10();
   default: return NULL;
   }
 }
@@ -474,51 +474,51 @@ ARKodeSPRKTable ARKodeSPRKTable_LoadByName(const char* method)
 {
   if (!strcmp(method, "ARKODE_SPRK_EULER_1_1"))
   {
-    return ARKodeSymplecticEuler();
+    return arkodeSymplecticEuler();
   }
   if (!strcmp(method, "ARKODE_SPRK_LEAPFROG_2_2"))
   {
-    return ARKodeSymplecticLeapfrog2();
+    return arkodeSymplecticLeapfrog2();
   }
   if (!strcmp(method, "ARKODE_SPRK_PSEUDO_LEAPFROG_2_2"))
   {
-    return ARKodeSymplecticPseudoLeapfrog2();
+    return arkodeSymplecticPseudoLeapfrog2();
   }
   if (!strcmp(method, "ARKODE_SPRK_RUTH_3_3"))
   {
-    return ARKodeSymplecticRuth3();
+    return arkodeSymplecticRuth3();
   }
   if (!strcmp(method, "ARKODE_SPRK_MCLACHLAN_2_2"))
   {
-    return ARKodeSymplecticMcLachlan2();
+    return arkodeSymplecticMcLachlan2();
   }
   if (!strcmp(method, "ARKODE_SPRK_MCLACHLAN_3_3"))
   {
-    return ARKodeSymplecticMcLachlan3();
+    return arkodeSymplecticMcLachlan3();
   }
   if (!strcmp(method, "ARKODE_SPRK_MCLACHLAN_4_4"))
   {
-    return ARKodeSymplecticMcLachlan4();
+    return arkodeSymplecticMcLachlan4();
   }
   if (!strcmp(method, "ARKODE_SPRK_CANDY_ROZMUS_4_4"))
   {
-    return ARKodeSymplecticCandyRozmus4();
+    return arkodeSymplecticCandyRozmus4();
   }
   if (!strcmp(method, "ARKODE_SPRK_MCLACHLAN_5_6"))
   {
-    return ARKodeSymplecticMcLachlan5();
+    return arkodeSymplecticMcLachlan5();
   }
   if (!strcmp(method, "ARKODE_SPRK_YOSHIDA_6_8"))
   {
-    return ARKodeSymplecticYoshida6();
+    return arkodeSymplecticYoshida6();
   }
   if (!strcmp(method, "ARKODE_SPRK_SUZUKI_UMENO_8_16"))
   {
-    return ARKodeSymplecticSuzukiUmeno816();
+    return arkodeSymplecticSuzukiUmeno816();
   }
   if (!strcmp(method, "ARKODE_SPRK_SOFRONIOU_10_36"))
   {
-    return ARKodeSymplecticSofroniou10();
+    return arkodeSymplecticSofroniou10();
   }
   return NULL;
 }
